"""Flipt gRPC client"""
import grpc
from flipt_pb2_grpc import FliptStub
from django.conf import settings


channel = grpc.insecure_channel(settings.FLIPT_GRPC_HOST)
client = FliptStub(channel=channel)



"""Flipt flag-related functions"""
from django.conf import settings
from flipt_pb2 import EvaluationRequest
from grpc import RpcError

from flipt.clients import client


def flag_enabled(flag_key: str, user=None) -> bool:
    override_value = settings.FLAG_OVERRIDDEN.get(flag_key)
    if override_value is not None:
        return bool(override_value)

    request = EvaluationRequest(
        flag_key=flag_key,
        entity_id=user.username if user and user.username else '_'
    )

    try:
        client.Evaluate(request)
    except RpcError:
        # TODO: catch error
        return False

    return True


def flag_disabled(flag_key: str, user=None) -> bool:
    return not flag_enabled(flag_key, user)



"""Flipt decorators"""
from functools import wraps
from typing import Dict

from django.http import Http404
from django.test import override_settings

from flipt.flags import flag_enabled


# pylint: disable=invalid-name
class override_flags(override_settings):
    def __init__(self, flags: Dict[str, bool]):
        flags_setting = {}

        for flag_key, value in flags.items():
            if isinstance(value, bool):
                flags_setting[flag_key] = value

        super().__init__(FLAG_OVERRIDDEN=flags_setting)


def flag_check(flag_key: str, state=True):
    def decorator(func):
        def inner(request, *args, **kwargs):
            enabled = flag_enabled(flag_key, user=request.user)

            if bool(state) == bool(enabled):
                return func(request, *args, **kwargs)
            raise Http404

        return wraps(func)(inner)

    return decorator



"""Django REST Framework views"""
from flipt_pb2 import ListFlagRequest
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from flipt.clients import client


class FeatureFlagListView(APIView):
    """View to retrieve all Flipt flags"""
    permission_classes = (AllowAny,)

    def get(self, _):
        response = client.ListFlags(ListFlagRequest())

        results = {}
        for flag in response.flags:
            results[flag.key] = flag.enabled

        return Response(results)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flipt', 'flipt.templatetags']

package_data = \
{'': ['*']}

install_requires = \
['django>=3.0.0,<4.0.0', 'flipt-grpc-python3>=0.1.0,<0.2.0']

extras_require = \
{'rest': ['djangorestframework>=3.12.2,<4.0.0']}

setup_kwargs = {
    'name': 'django-flipt',
    'version': '0.1.1',
    'description': 'Flipt Integration for Django and Django REST Framework',
    'long_description': "# django-flipt\nFlipt Integration for Django and Django REST Framework\n\n## Installation\n\n```shell\npip install django-flipt\n```\n\n## Usage\n\n1. Add `flipt` into `INSTALLED_APPS`\n\n```python\nINSTALLED_APPS = [\n    ...\n    'flipt',\n]\n```\n\n2. Define Flipt gRPC endpoint in `settings.py`\n\n```python\nFLIPT_GRPC_HOST = 'flipt:9000'\n```\n\n3. Ready to go!\n\n### Overriding Flags\n\nYou can override any flag by defining your flag key and overriding value\n\n```python\nFLAG_OVERRIDDEN = {\n    'some-flag-key': True\n}\n```\n\n### Available Classes/Functions\n\n- `flag_enabled`\n- `flag_disabled`\n- `FlaggedRouter`\n- `@flag_check`\n- `@override_flags`\n- `{% featureflag %} ... {% endfeatureflag %}`\n- `FeatureFlagListView`\n\n## Development\n\n### Requirements\n\n- Docker\n\n### Run Project\n\n```shell\n$ make\n```\n\n### Linting/Test Project\n\n```shell\n$ make lint\n$ make test\n```\n",
    'author': 'Preeti Yuankrathok',
    'author_email': 'preetisatit@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/earthpyy/django-flipt',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



"""
Usage:
{% featureflag <flag_key> [<flag_state=True>] %}
    ...
{% endfeatureflag %}
"""
from django import template

from flipt.flags import flag_enabled

register = template.Library()


@register.tag
def featureflag(parser, token):
    nodelist = parser.parse(('endfeatureflag',))
    parser.delete_first_token()

    try:
        _, params = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            '%r tag requires one or two arguments' % token.contents.split()[0]
        ) from ValueError

    params = params.split()
    flag_key = str(params[0]).rstrip('"').lstrip('"')
    state = params[1] if len(params) > 1 else True

    return FeatureFlagNode(nodelist, flag_key, state)


class FeatureFlagNode(template.Node):
    def __init__(self, nodelist, flag_key: str, state: bool):
        self.nodelist = nodelist
        self.flag_key = flag_key
        self.state = state

    def render(self, context):
        should_render = self.state == flag_enabled(self.flag_key)
        return self.nodelist.render(context) if should_render else ''



"""Flipt router for Django REST Framework"""
from rest_framework.routers import DefaultRouter

from flipt.decorators import flag_check


class FlaggedRouter(DefaultRouter):
    """
    Usage:

    router = FlaggedRouter()
    router.register('test', TestViewSet, flag_key='flag_key', flag_state=True)

    * `flag_state` is not required
    """
    flagged_infos = {}

    def register(self, prefix, viewset, basename=None, flag_key: str = None,
                 flag_state=True):
        super().register(prefix, viewset, basename=basename)

        if flag_key:
            self.flagged_infos[viewset] = {
                'flag_key': flag_key,
                'state': flag_state
            }

    def get_urls(self):
        urls = super().get_urls()
        for url in urls:
            flagged_info = self.flagged_infos.get(url.callback.cls)
            if not flagged_info:
                continue

            flagged_view = flag_check(**flagged_info)(url.callback)
            url.callback = flagged_view

        return urls
